from aiogram import Bot
from aiogram.types import BotCommand


async def register_my_commands(bot: Bot):
    return await bot.set_my_commands(
        [
            BotCommand(
                command="start",
                description="Starts bot"
            )
        ]
    )



from aiogram import Bot, Dispatcher

"""  
Public Scripts to use it for any routers,  
or to call it from other pieces of structure,  
like another code (not tgbot)
"""

bot: Bot
dp: Dispatcher


def register_scripts(work_bot: Bot, work_dp: Dispatcher):
    global bot, dp

    bot = work_bot
    dp = work_dp


__all__ = ['register_scripts', 'bot', 'dp']



import importlib
import os
import yoyo

from aiogram import Router

from .middlewares import register_middlewares
from .handlers import register_handlers


def get_router() -> tuple[int, Router]:
    router = Router(name=yoyo.tools.basename(__file__))

    # Register middlewares, handlers and errors
    register_middlewares(router)
    register_handlers(router)

    # Include to router another routers
    path = yoyo.tools.listpath(__file__)
    path.append("routers")
    try:
        routers = os.listdir("\\".join(path))
        package = yoyo.tools.package(path)
        for module in routers:
            import_get_router = importlib.import_module(f".{module}", package=package).get_router
            router.include_router(import_get_router())
    except (FileNotFoundError, TypeError):
        pass

    return 1, router



import sys
import os
import logging

from datetime import datetime


class Logger(object):
    log_file = bytes
    EXCEPTION = 100
    CRITICAL = 50
    ERROR = 40
    WARNING = 30
    INFO = 20
    DEBUG = 10
    NOTSET = 0

    def __init__(self):
        today = datetime.strftime(datetime.now(), "%d-%m-%y-%H_%M_%S")

        if not os.path.exists("logs"):
            os.mkdir("logs")

        LOGS_FOLDER = f"logs/log-{today}.log"
        LOGGER_NAME = "YoYoLogger"

        logging.basicConfig(
            filename=LOGS_FOLDER,
            filemode="w",
            format="[%(asctime)s] [%(levelname)s] -- %(message)s",
            datefmt="%d/%b/%y %H:%M:%S",
            encoding="utf-8"
        )


        self.__log = logging.getLogger(LOGGER_NAME)
        formatter = logging.Formatter(
            "[%(asctime)s] [%(levelname)s] -- %(message)s", datefmt="%d/%b/%y %H:%M:%S"
        )
        ch = logging.StreamHandler(sys.stderr)
        ch.setFormatter(formatter)

        self.__log.setLevel(self.INFO)
        self.__methods_map = {
            self.DEBUG: self.__log.debug,
            self.INFO: self.__log.info,
            self.WARNING: self.__log.warning,
            self.ERROR: self.__log.error,
            self.CRITICAL: self.__log.critical,
            self.EXCEPTION: self.__log.exception,
        }


    def __call__(self, lvl, msg, *args, **kwargs):
        if lvl in self.__methods_map:
            self.__methods_map[lvl](msg, *args, **kwargs)
        else:
            self.__log.log(lvl, msg, *args, **kwargs)


    def set_level(self, level=None):
        if level is None:
            level = self.INFO
        self.__log.setLevel(level)



import importlib
from src import yoyo

from aiogram import Bot
from apscheduler.schedulers.asyncio import AsyncIOScheduler

from tgbot.bots.{bot_name}.data.config import get_config

scheduler: AsyncIOScheduler


def register_scheduler(bot: Bot):
    global scheduler
    scheduler = AsyncIOScheduler(timezone=get_config().misc.tz)

    tasks = list(map(lambda x: x[:-3], yoyo.tools.listdir(__file__)))
    tasks.remove('__init__')
    tasks.remove('__pycach')

    package = yoyo.tools.package(__file__)
    for module in tasks:
        import_register = importlib.import_module(f'.{module}', package=package).register
        import_register(scheduler, bot)

    scheduler.start()


DEFAULT_LANGUAGE = 'en'

FIRST_TEXT = {
	'en': "Hello!",
	'ru': "Привет!"
}

def get_text(lang_code: str, text: dict[str, str]) -> str:
    if not lang_code in text.keys():
        lang_code = DEFAULT_LANGUAGE
    return text[lang_code]



import asyncio
from asyncio import CancelledError

from aiogram import Bot, Dispatcher
from aiogram.fsm.storage.memory import MemoryStorage
from aiogram.fsm.storage.redis import RedisStorage
from redis import Redis

from tgbot.bots import build_bots, start_bots


async def build() -> list[tuple[Bot, Dispatcher]]:
    return await build_bots()


async def start(bots: list[tuple[Bot, Dispatcher]]):
    await start_bots(bots)


async def main():
    await start(await build())


from setuptools import setup, find_packages

setup(
    name="yoyogram",
    version="0.0.1",
    description="The framework to ease the work with huge projects with aiogram",
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    author="dhmmmhb",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "yoyo = _yoyo_cli.cli:main"
        ]
    },
    install_requires=[
        "aiogram",
        "redis",
        "environs",
        "aiosqlite",
        "APScheduler",
        "rich"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.7",
)



import shutil
from pathlib import Path
import importlib.resources as pkg_resources

from rich.console import Console
from rich.table import Table

from _yoyo_cli import templates


def get_project_root():
    current = Path.cwd()
    for parent in [current] + list(current.parents):
        if (parent / "main.py").exists() or (parent / ".git").exists():
            return parent
    return current


def init_project(console: Console):
    dst_path = Path.cwd()
    with pkg_resources.path(templates, "project") as template_path:
        shutil.copytree(template_path, dst_path, dirs_exist_ok=True)
        (dst_path / "__init__.py").unlink(missing_ok=True)
        console.print("[green]✅ Project has been created![/green]")


def help_command(console: Console):
    table = Table(title="🛠 Доступные команды YoYoGram CLI", style="magenta")
    table.add_column("Command", style="cyan3", no_wrap=True)
    table.add_column("Arguments", style="deep_sky_blue1")
    table.add_column("Description", style="white")
    table.add_row("init", "", "Create a new project")
    table.add_row("bot", "bot_name", "Create a new bot")
    table.add_row("router", "name Optional:index Optional:bot_name\nProviding bot_name it will create router by tgbot/bots/bot_name/routers/router_name path\nNot providing bot_name, it established router by the current path\nNot providing index means the last position", "Establish a new router")
    console.print(table)


def create_bot(console: Console, bot_name: str):
    with pkg_resources.path(templates, "bot") as template_path:
        bots_dir = get_project_root() / "tgbot" / "bots" / bot_name

        for item in template_path.rglob("*"):
            if item.is_file():
                rel_path = item.relative_to(template_path)
                dst_file = bots_dir / rel_path
                dst_file.parent.mkdir(parents=True, exist_ok=True)

                try:
                    content = item.read_text(encoding="utf-8")
                    content = content.replace("{bot_name}", bot_name)
                    dst_file.write_text(content, encoding="utf-8")
                except UnicodeDecodeError:
                    shutil.copyfile(item, dst_file)

        console.print(f"[green]✅ {bot_name} has been created by tgbot/bots/{bot_name} directory![/green]")


def establish_router(console: Console, router_name: str, router_index: int = -1, bot_name: str = None):
    with pkg_resources.path(templates, "router") as template_path:
        if bot_name:
            bots_dir = get_project_root() / "tgbot" / "bots" / bot_name / "routers" / router_name
        else:
            for parent in [Path.cwd()] + list(Path.cwd().parents):
                if (parent / "routers").exists():
                    bots_dir = parent / "routers" / router_name
                    break
            else:
                bots_dir = get_project_root() / "tgbot" / "bots" / bot_name / "routers" / router_name

        for item in template_path.rglob("*"):
            if item.is_file():
                rel_path = item.relative_to(template_path)
                dst_file = bots_dir / rel_path
                dst_file.parent.mkdir(parents=True, exist_ok=True)
                try:
                    content = item.read_text(encoding="utf-8")
                    content = content.replace("{bot_name}", bot_name)
                    content = content.replace("{router_name}", router_name)
                    content = content.replace("{router_index}", str(router_index))
                    dst_file.write_text(content, encoding="utf-8")
                except UnicodeDecodeError:
                    shutil.copyfile(item, dst_file)

        if bot_name:
            console.print(f"[green]✅ {router_name} has been established by tgbot/bots/{bot_name}/routers/{router_name} directory![/green]")
        else:
            console.print(f"[green]✅ {router_name} has been established by {str(bots_dir)} directory![/green]")
